-- Create the addon object
WhiteboardNG = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Metrognome-2.0")

local dewdrop = AceLibrary("Dewdrop-2.0")
local compost = AceLibrary("Compost-2.0")

WhiteboardNG.version = "0.6.0"

local commPrefix = "WhiteboardNG"

WhiteboardNG:RegisterDB("WhiteboardNGDB")
WhiteboardNG:RegisterDefaults("profile", {
	master = "<off>",
	sync_enable = false,
	drawings = {},
})

WhiteboardNG:RegisterChatCommand({"/whiteboardng", "/wbng"}, {
	type = "group",
	name = "WhiteboardNG",
	args = {
		show = {
			name = "Show",
			desc = "Show the whiteboard",
			type = "execute",
			func = function()
				if not WhiteboardNG.board then
					WhiteboardNG:DrawBoard()
				end
				WhiteboardNG.board:Show()
			end,
		},
	},
}, "WHITEBOARDNG")

local frames = {
	text = {},
	icons = {}
}
local dewdropmenu = {}
local textmenu = {}
local curtextmenu

function WhiteboardNG:OnInitialize()
	-- Register Metrognome timers
	self:RegisterMetro("Whiteboard_Update", self.OnUpdate, 0.01, self)
	self:RegisterMetro("Whiteboard_DataQueueStatus", self.DataQueueStatus, 1, self)
	self:RegisterMetro("Whiteboard_ProcessQueue", self.ProcessQueue, 0.2, self)
	self:RegisterMetro("Whiteboard_CheckDataReceived", self.CheckDataReceived, 1, self)

	-- Set starting states
	self.optColor = {0, 0, 0, 1}
	self.Drawing = false
	self.Erasing = false
	self.dataReceived = false
	self.dataQueue = compost:Acquire()
	self.syncData = compost:Acquire()

	self:SyncEnabled(self.db.profile.sync_enable)

	-- Initialize AceComm
	self:SetCommPrefix(commPrefix)
	self:RegisterMemoizations({	"DRAWLINE", "ERASE", "CLEAR",
								"SETTEXT", "REMTEXT", "SETICON", "REMICON",
								"SETMASTER", "QSETMASTER", "REQMASTER", "VERSION", "VERSIONREPLY",
								"SYNCSTART", "SYNCSTOP"})

	-- Register Dewdrop menu
end
function WhiteboardNG:DrawBoard()
	-- Create the whiteboard frame
	self.board = CreateFrame("Frame", "WBNGFrame", UIParent)
	self.board:SetWidth(640)
	self.board:SetHeight(480)
	self.board:SetPoint("BOTTOMLEFT", self.db.profile.mainx or UIParent:GetWidth()/4, self.db.profile.mainy or UIParent:GetHeight()/4)
	self.board:SetClampedToScreen(true)
	self.board:SetMovable(true)
	self.board:EnableMouse(true)
	self.board:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
	self.board:SetBackdropColor(1.0, 1.0, 1.0, 1)
	self.board:SetBackdrop({bgFile = "Interface\\Addons\\WhiteboardNG\\textures\\PlainBackdrop.tga",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true, tileSize = 16, edgeSize = 16,
							insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.board:SetScript("OnMouseDown", function()
		if (((not this.isLocked) or (this.isLocked == 0)) and (arg1 == "LeftButton") and IsShiftKeyDown()) then
			this:StartMoving()
			this.isMoving = true
		end
		if(not IsShiftKeyDown() and not IsControlKeyDown() and (arg1 == "LeftButton")) then
			WhiteboardNG.Drawing=true
		end
		if(IsControlKeyDown() and (arg1 == "LeftButton")) then
			WhiteboardNG.Erasing=true
		end
	end)
	self.board:SetScript("OnMouseUp", function()
		if(this.isMoving) then
			this:StopMovingOrSizing()
			this.isMoving = false
		end
		WhiteboardNG.Drawing = false
		WhiteboardNG.Erasing = false
		WhiteboardNG.db.profile.mainx = WhiteboardNG.board:GetLeft()
		WhiteboardNG.db.profile.mainy = WhiteboardNG.board:GetBottom()
	end)
	self.board:SetScript("OnHide", function()
		WhiteboardNG:StopMetro("Whiteboard_Update")
		WhiteboardNG:StopMetro("Whiteboard_ProcessQueue")
		if(this.isMoving) then
			this:StopMovingOrSizing()
			this.isMoving = false
		end
		dewdrop:Close()
		WhiteboardNG.dataReceived = false
		WhiteboardNG.versionQuery = false
		if(WhiteboardNG.versionFrame) then
			WhiteboardNG.versionFrame:Hide()
		end
	end)
	self.board:SetScript("OnShow", function()
		WhiteboardNG:StartMetro("Whiteboard_Update")
		WhiteboardNG:StartMetro("Whiteboard_ProcessQueue")
		WhiteboardNG.dataReceived = false
	end)
	self.board:Hide()

	self.mini = CreateFrame("Frame", "WBNGMiniFrame", UIParent)
	self.mini:SetWidth(200)
	self.mini:SetHeight(32)
	self.mini:SetPoint("BOTTOMLEFT", self.db.profile.minix or UIParent:GetWidth()/2, self.db.profile.miniy or UIParent:GetHeight()/2)
	self.mini:SetClampedToScreen(true)
	self.mini:SetMovable(true)
	self.mini:EnableMouse(true)
	self.mini:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
								edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
								tile = true, tileSize = 16, edgeSize = 16,
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.mini:SetBackdropColor(0, 0, 0, 0.5)
	self.mini:SetScript("OnMouseDown", function()
		if(IsShiftKeyDown()) then
			this:StartMoving()
			this.isMoving = true
		end
	end)
	self.mini:SetScript("OnMouseUp", function()
		if(this.isMoving) then
			this:StopMovingOrSizing()
			this.isMoving = false
		end
		WhiteboardNG.db.profile.minix = WhiteboardNG.board:GetLeft()
		WhiteboardNG.db.profile.miniy = WhiteboardNG.board:GetBottom()
	end)

	self.mini.text = self.mini:CreateFontString(nil, "OVERLAY")
	self.mini.text:SetWidth(150)
	self.mini.text:SetHeight(16)
	self.mini.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.mini.text:SetTextColor(1, 1, 0, 1)
	self.mini.text:SetPoint("TOPLEFT", 10, -8)
	self.mini.text:SetJustifyH("LEFT")
	self.mini.text:SetText("WhiteboardNG")

	self.mini.flash = self.mini:CreateTexture(nil, "BACKGROUND")
	self.mini.flash:SetWidth(205)
	self.mini.flash:SetHeight(42)
	self.mini.flash:SetTexture("Interface\\AddOns\\WhiteboardNG\\textures\\Flash")
	self.mini.flash:SetTexCoord(0, 0.82, 0, 1)
	self.mini.flash:SetPoint("TOPLEFT", -3, 5)
	self.mini.flash:Hide()

	self.mini.close = CreateFrame("Button", nil, self.mini, "UIPanelCloseButton")
	self.mini.close:SetPoint("TOPRIGHT", 0, 0)
	self.mini.close:SetScript("OnClick", function()
		this:GetParent():Hide()
	end)

	self.mini.maximize = CreateFrame("Button", nil, self.mini)
	self.mini.maximize:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-UP")
	self.mini.maximize:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-DOWN")
	self.mini.maximize:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-HILIGHT")
	self.mini.maximize:SetWidth(24)
	self.mini.maximize:SetHeight(24)
	self.mini.maximize:SetPoint("TOPRIGHT", -32, -4)
	self.mini.maximize:SetScript("OnClick", function()
		this:GetParent():Hide()
		WhiteboardNG.board:Show()
	end)

	self.mini:Hide()

	self.syncFrame = CreateFrame("Frame", nil, self.board)
	self.syncFrame:SetWidth(220)
	self.syncFrame:SetHeight(30)
	self.syncFrame:SetPoint("BOTTOMLEFT", self.board, "TOPLEFT", 0, 0)
	self.syncFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
								edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
								tile = true, tileSize = 16, edgeSize = 16,
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.syncFrame:SetBackdropColor(0, 0, 0, 0.5)

	self.syncFrame.text = self.syncFrame:CreateFontString(nil, "OVERLAY")
	self.syncFrame.text:SetWidth(180)
	self.syncFrame.text:SetHeight(16)
	self.syncFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.syncFrame.text:SetTextColor(1, 1, 1, 1)
	self.syncFrame.text:SetPoint("TOPLEFT", 10, -7)
	self.syncFrame.text:SetJustifyH("LEFT")
	self.syncFrame.text:SetText("Objects in sync-data:")

	self.syncFrame.count = self.syncFrame:CreateFontString(nil, "OVERLAY")
	self.syncFrame.count:SetWidth(40)
	self.syncFrame.count:SetHeight(16)
	self.syncFrame.count:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.syncFrame.count:SetTextColor(1, 1, 1, 1)
	self.syncFrame.count:SetPoint("TOPRIGHT", -10, -7)
	self.syncFrame.count:SetJustifyH("RIGHT")
	self.syncFrame.count:SetText("0")

	self.dataFrame = CreateFrame("Frame", nil, self.board)
	self.dataFrame:SetWidth(220)
	self.dataFrame:SetHeight(30)
	self.dataFrame:SetPoint("TOPLEFT", self.syncFrame, "TOPRIGHT", 0, 0)
	self.dataFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
								edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
								tile = true, tileSize = 16, edgeSize = 16,
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.dataFrame:SetBackdropColor(0, 0, 0, 0.5)

	self.dataFrame.text = self.dataFrame:CreateFontString(nil, "OVERLAY")
	self.dataFrame.text:SetWidth(180)
	self.dataFrame.text:SetHeight(16)
	self.dataFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.dataFrame.text:SetTextColor(1, 1, 1, 1)
	self.dataFrame.text:SetPoint("TOPLEFT", 10, -7)
	self.dataFrame.text:SetJustifyH("LEFT")
	self.dataFrame.text:SetText("Objects in data queue:")

	self.dataFrame.count = self.dataFrame:CreateFontString(nil, "OVERLAY")
	self.dataFrame.count:SetWidth(40)
	self.dataFrame.count:SetHeight(16)
	self.dataFrame.count:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.dataFrame.count:SetTextColor(1, 1, 1, 1)
	self.dataFrame.count:SetPoint("TOPRIGHT", -10, -7)
	self.dataFrame.count:SetJustifyH("RIGHT")
	self.dataFrame.count:SetText("0")


	self.masterFrame = CreateFrame("Frame", nil, self.board)
	self.masterFrame:SetWidth(200)
	self.masterFrame:SetHeight(30)
	self.masterFrame:SetPoint("TOPLEFT", self.dataFrame, "TOPRIGHT", 0, 0)
	self.masterFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
								edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
								tile = true, tileSize = 16, edgeSize = 16,
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.masterFrame:SetBackdropColor(0, 0, 0, 0.5)

	self.masterFrame.text = self.masterFrame:CreateFontString(nil, "OVERLAY")
	self.masterFrame.text:SetWidth(80)
	self.masterFrame.text:SetHeight(16)
	self.masterFrame.text:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.masterFrame.text:SetTextColor(1, 1, 1, 1)
	self.masterFrame.text:SetPoint("TOPLEFT", 10, -7)
	self.masterFrame.text:SetJustifyH("LEFT")
	self.masterFrame.text:SetText("Master:")

	self.masterFrame.count = self.masterFrame:CreateFontString(nil, "OVERLAY")
	self.masterFrame.count:SetWidth(120)
	self.masterFrame.count:SetHeight(16)
	self.masterFrame.count:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	self.masterFrame.count:SetTextColor(1, 1, 1, 1)
	self.masterFrame.count:SetPoint("TOPRIGHT", -10, -7)
	self.masterFrame.count:SetJustifyH("RIGHT")
	self.masterFrame.count:SetText(self.db.profile.master)


	self.versionFrame = CreateFrame("Frame", nil, self.board)
	self.versionFrame:SetWidth(200)
	self.versionFrame:SetHeight(480)
	self.versionFrame:SetPoint("TOPRIGHT", self.board, "TOPLEFT", 0, 0)
	self.versionFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
								edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
								tile = true, tileSize = 16, edgeSize = 16,
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
	self.versionFrame:SetBackdropColor(0, 0, 0, 0.5)

	f, _, _ = GameFontNormal:GetFont()

	self.versionFrame.header = self.versionFrame:CreateFontString(nil,"OVERLAY")
	self.versionFrame.header:SetFont(f, 14)
	self.versionFrame.header:SetWidth(170)
	self.versionFrame.header:SetText("Version Query")
	self.versionFrame.header:SetTextColor(1, .8, 0)
	self.versionFrame.header:ClearAllPoints()
	self.versionFrame.header:SetJustifyH("LEFT")
	self.versionFrame.header:SetPoint("TOPLEFT", 5, -5)

	self.versionFrame.headername = self.versionFrame:CreateFontString(nil,"OVERLAY")
	self.versionFrame.headername:SetFont(f, 14)
	self.versionFrame.headername:SetWidth(115)
	self.versionFrame.headername:SetText("Name")
	self.versionFrame.headername:SetTextColor(1, .8, 0)
	self.versionFrame.headername:ClearAllPoints()
	self.versionFrame.headername:SetJustifyH("LEFT")
	self.versionFrame.headername:SetPoint("TOPLEFT", 10, -25)

	self.versionFrame.headerversion = self.versionFrame:CreateFontString(nil,"OVERLAY")
	self.versionFrame.headerversion:SetFont(f, 14)
	self.versionFrame.headerversion:SetWidth(75)
	self.versionFrame.headerversion:SetText("Version")
	self.versionFrame.headerversion:SetTextColor(1, .8, 0)
	self.versionFrame.headerversion:ClearAllPoints()
	self.versionFrame.headerversion:SetJustifyH("LEFT")
	self.versionFrame.headerversion:SetPoint("TOPLEFT", self.versionFrame.headername, "TOPRIGHT")

	self.versionFrame.entry = {}
	self.versionFrame.entry[1] = self:ScrollEntryFrame(1)
	self.versionFrame.entry[1]:SetPoint("TOPLEFT", self.versionFrame, "TOPLEFT", 10, -40 )

	for i=2, 27 do
		j = i - 1
		self.versionFrame.entry[i] = self:ScrollEntryFrame()
		self.versionFrame.entry[i]:SetPoint("TOPLEFT", self.versionFrame.entry[j], "BOTTOMLEFT")
	end

	self.versionFrame.sframe = CreateFrame("ScrollFrame", "WBNGFrame_Scroll", self.versionFrame, "FauxScrollFrameTemplate")
	self.versionFrame.sframe.owner = self
	self.versionFrame.sframe:SetWidth(165)
	self.versionFrame.sframe:SetHeight(430)
	self.versionFrame.sframe:SetPoint("TOPLEFT", self.versionFrame, "TOPLEFT", 5, -40)

	self.versionFrame.sframe:SetScript("OnVerticalScroll",
			function()
				FauxScrollFrame_OnVerticalScroll(28, function() this.owner:UpdateScrollBar() end)
			end )

	self.versionFrame.close = CreateFrame("Button", nil, self.versionFrame, "UIPanelCloseButton")
	self.versionFrame.close:SetPoint("TOPRIGHT", 0, 0)
	self.versionFrame.close:SetScript("OnClick", function()
		WhiteboardNG.versionQuery = false
		this:GetParent():Hide()
	end)

	self.versionFrame:Hide()
	dewdrop:Register(self.board,
		"children", function()
			dewdrop:FeedAceOptionsTable(dewdropmenu)
		end,
		"cursorX", true,
		"cursorY", true
	)
end
function WhiteboardNG:OnEnable()
	-- Register Comm stream
	self:RegisterComm(commPrefix, "GROUP")

	-- Start update timers
	self:StartMetro("Whiteboard_DataQueueStatus")
	self:StartMetro("Whiteboard_CheckDataReceived")

	if(self:InGroup() and not self:IsLeader(true)) then
		self:SendData({"REQMASTER"}, "ALERT")
	elseif(not self:InGroup()) then
		self.db.profile.master = "<off>"
	end

	self:RegisterEvent("RAID_ROSTER_UPDATE", 	"UpdateMaster")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateMaster")

end

function WhiteboardNG:OnUpdate()
	if(self.syncInProgress) then return end

	if self.Drawing then
		local X, Y = GetCursorPosition()
		X = (X / self.board:GetEffectiveScale() - self.board:GetLeft())
		Y = (Y / self.board:GetEffectiveScale() - self.board:GetBottom())
		if not self:OnBoard(X, Y) then
			self.LastX=nil
			self.LastY=nil
			return
		end
		if self.LastX and self.LastY then
			local Dist=math.abs(X-self.LastX)+math.abs(Y-self.LastY)
			if Dist<=4 then
				return
			end
			if(self:AllowedDraw()) then
				IMBA_DrawLine(self.board,self.LastX,self.LastY,X,Y,32,self.optColor)
				local args, i = {}, 1
				for id in string.gmatch(string.format("%.1f %.1f %.1f %.1f",self.LastX,self.LastY,X,Y), "%S+") do
					args[i] = id
					i = i + 1
				end
				self:SendData({"DRAWLINE", args[1], args[2], args[3], args[4], self.optColor})
			end
		end
		self.LastX=X
		self.LastY=Y
		self.LastEX = nil
		self.LastEY = nil
	else
		if self.Erasing then
			local X, Y = GetCursorPosition()
			X = (X / self.board:GetEffectiveScale() - self.board:GetLeft())
			Y = (Y / self.board:GetEffectiveScale() - self.board:GetBottom())
			if not self:OnBoard(X, Y) then
				self.LastEX=nil
				self.LastEY=nil
				return
			end
			if self.LastEX and self.LastEY then
				local Dist=math.abs(X-self.LastEX)+math.abs(Y-self.LastEY)
				if Dist<=4 then
					return
				end
				if(self:AllowedDraw()) then
					IMBA_EraseLines(self.board,X,Y,12)
					local args, i = {}, 1
					for id in string.gmatch(string.format("%.1f %.1f",X,Y), "%S+") do
						args[i] = id
						i = i + 1
					end
					self:SendData({"ERASE", args[1], args[2]})
				end
			end

			self.LastEX = X
			self.LastEY = Y
		end
		self.LastX=nil
		self.LastY=nil
	end
end

function WhiteboardNG:ClearBoard(recv)
	if(self.syncInProgress and not recv) then return end

	if(self:AllowedDraw() or recv) then
		self.dataQueue = compost:Erase(self.dataQueue)
		IMBA_ClearLines(self.board)
		for k,v in pairs(self.Icons) do
			self:RemoveIcon(k, true)
		end
		for k,v in pairs(frames.text) do
			self:RemoveText(k, true)
		end
		if(not recv) then
			self:SendData({"CLEAR"})
		end
		self:ClearSync()
	end
end

function WhiteboardNG:SetIcon(id, x, y, recv)
	if(not recv and not WhiteboardNG:AllowedDraw()) then return end
	if(self.syncInProgress and not recv) then return end

	local icon = self.Icons[id]
	if(not frames.icons[icon.ID]) then
		frames.icons[icon.ID] = {}

		local f = CreateFrame("Frame", nil, self.board)
		f:SetWidth(32)
		f:SetHeight(32)
		f:EnableMouse(true)
		f.moving = false
		f.id = id
		f.active = true
		f:SetScript("OnMouseDown", function()
			if(not WhiteboardNG:AllowedDraw()) then return end
			if(arg1 == "LeftButton") then
				this.moving = true
			elseif(arg1 == "RightButton") then
				WhiteboardNG:RemoveIcon(this.id)
			end
		end)
		f:SetScript("OnMouseUp", function()
			if(this.moving) then
				WhiteboardNG:SendData({"SETICON", this.id, this:GetLeft()-this:GetParent():GetLeft(), this:GetBottom()-this:GetParent():GetBottom()})
			end
			this.moving = false
		end)
		f:SetScript("OnUpdate", function()
			if(this.moving) then
				local d = this:GetWidth() / 2
				local X, Y = GetCursorPosition()
				X = (X / this:GetParent():GetEffectiveScale() - this:GetParent():GetLeft())
				Y = (Y / this:GetParent():GetEffectiveScale() - this:GetParent():GetBottom())

				if(X < d) then
					X = d
				elseif(X > this:GetParent():GetWidth()-d) then
					X = this:GetParent():GetWidth()-d
				end
				if(Y < d) then
					Y = d
				elseif(Y > this:GetParent():GetHeight()-d) then
					Y = this:GetParent():GetHeight()-d
				end

				this:ClearAllPoints()
				this:SetPoint("BOTTOMLEFT", X-d, Y-d)
			end
		end)

		f.texture = f:CreateTexture(nil, "BACKGROUND")
		f.texture:SetTexture(icon.Texture)
		f.texture:SetWidth(32)
		f.texture:SetHeight(32)
		f.texture:SetAllPoints(true)

		if(x == -1 and y == -1) then
			f:SetPoint("CENTER", 0, 0)
		else
			f:SetPoint("BOTTOMLEFT", x, y)
		end

		f:Show()
		frames.icons[icon.ID] = f
	else
		local f = frames.icons[icon.ID]
		f:ClearAllPoints()
		if(x == -1 and y == -1) then
			f:SetPoint("CENTER", 0, 0)
		else
			f:SetPoint("BOTTOMLEFT", x, y)
		end
		f:Show()
	end
	if(not recv) then
		self:SendData({"SETICON", id, x, y})
	end
end

function WhiteboardNG:RemoveIcon(id, recv)
	if(self.syncInProgress and not recv) then return end

	if(frames.icons[self.Icons[id].ID]) then
		frames.icons[self.Icons[id].ID].active = false
		frames.icons[self.Icons[id].ID]:Hide()
	end

	if(not recv) then
		self:SendData({"REMICON", id})
	end
end

function WhiteboardNG:SetText(id, ping, x, y, justify, color, text, recv)
	if(self.syncInProgress and not recv) then return end

	if(id == -1 or id > 0 and not frames.text[id]) then
		local i = 0
		if(id == -1) then
			for k,v in pairs(frames.text) do
				if(not v.active) then
					i = k
					break
				end
			end
			if(i == 0) then
				for k,v in pairs(frames.text) do
					if(k>i) then
						i = k
					end
				end
				i = i + 1
			end
		else
			i = id
		end

		if(not frames.text[i]) then
			local f = CreateFrame("Frame", nil, self.board)
			f:SetWidth(12)
			f:SetHeight(12)
			f:EnableMouse(true)
			f.moving = false
			f.id = i
			f:SetScript("OnMouseDown", function()
				if(not WhiteboardNG:AllowedDraw()) then return end

				if(arg1 == "LeftButton") then
					this.moving = true
				elseif(arg1 == "RightButton") then
					curtextmenu = this
					dewdrop:Open(this)
				end
			end)
			f:SetScript("OnMouseUp", function()
				if(this.moving) then
					WhiteboardNG:SendText(this.id)
				end
				this.moving = false
			end)
			f:SetScript("OnUpdate", function()
				if(this.moving) then
					local d = this:GetWidth() / 2
					local X, Y = GetCursorPosition()
					X = (X / this:GetParent():GetEffectiveScale() - this:GetParent():GetLeft())
					Y = (Y / this:GetParent():GetEffectiveScale() - this:GetParent():GetBottom())

					if(X < d) then
						X = d
					elseif(X > this:GetParent():GetWidth()-d) then
						X = this:GetParent():GetWidth()-d
					end
					if(Y < d) then
						Y = d
					elseif(Y > this:GetParent():GetHeight()-d) then
						Y = this:GetParent():GetHeight()-d
					end

					this:ClearAllPoints()
					this:SetPoint("BOTTOMLEFT", X-d, Y-d)
				end
			end)

			f.texture = f:CreateTexture(nil, "BACKGROUND")
			f.texture:SetTexture("Interface\\Minimap\\ObjectIcons")
			f.texture:SetAllPoints(true)

			f.txt = f:CreateFontString(nil, "OVERLAY")
			f.txt:SetHeight(16)
			f.txt:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")

			dewdrop:Register(f,
				"children", function()
					dewdrop:FeedAceOptionsTable(textmenu)
				end
			)
			frames.text[i] = f
		end
		id = i
	end

	local f = frames.text[id]

	f:Show()
	f.active = true

	f.ping = (ping or (math.floor(math.random()*4) + 1))
	f.texture:SetTexCoord(unpack(self.pingIcons[f.ping]))

	f.txt:SetText(text)
	f.txt:SetJustifyH(justify or "LEFT")

	local r, g, b, a = 1, 1, 1, 1
	if(color) then
		r, g, b, a = unpack(color)
	end
	f.txt:SetTextColor(r, g, b, a)

	f.txt:ClearAllPoints()
	if(justify) then
		if(justify == "LEFT") then
			f.txt:SetPoint("BOTTOMLEFT", f, "TOPRIGHT")
		elseif(justify == "CENTER") then
			f.txt:SetPoint("CENTER", f, "TOPLEFT", f:GetWidth()/2, 12)
		elseif(justify == "RIGHT") then
			f.txt:SetPoint("BOTTOMRIGHT", f, "TOPLEFT")
		end
	else
		f.txt:SetPoint("BOTTOMLEFT", f, "TOPRIGHT")
	end

	f:ClearAllPoints()
	if(x == -1 and y == -1) then
		f:SetPoint("CENTER", 0, 0)
	else
		f:SetPoint("BOTTOMLEFT", x, y)
	end

	if(not recv) then
		self:SendText(id)
	end
end

function WhiteboardNG:RemoveText(id, recv)
	if(self.syncInProgress and not recv) then return end

	if(frames.text[id]) then
		frames.text[id].active = false
		frames.text[id]:Hide()
	end

	if(not recv) then
		self:SendData({"REMTEXT", id})
	end
end

function WhiteboardNG:SendText(id)
	local f = frames.text[id]
	local r, g, b, a = f.txt:GetTextColor()
	local x, y = f:GetLeft() - f:GetParent():GetLeft(), f:GetBottom() - f:GetParent():GetBottom()
	local text = f.txt:GetText()
	local justify = f.txt:GetJustifyH()

	self:SendData({"SETTEXT", id, f.ping, {x, y}, justify, {r, g, b, a}, text})
end

function WhiteboardNG:SetMaster(opt, recv, quiet)
	self.db.profile.master = string.lower(opt)
	if(not recv) then
		self:SendData({"SETMASTER", self.db.profile.master}, "ALERT")
	end
	if(not quiet) then
		self:Print("Master set to: "..string.gsub(self.db.profile.master, "^%l", string.upper))
	end
	self.masterFrame.count:SetText(string.gsub(self.db.profile.master, "^%l", string.upper))
end

function WhiteboardNG:UpdateMaster()
	if(not self.nextUpdate or self.nextUpdate < GetTime()) then
		if(self:InGroup()) then
			self:SendData({"REQMASTER"}, "ALERT")
		else
			self.db.profile.master = "<off>"
		end
		self.nextUpdate = GetTime() + 10
	end
end

function WhiteboardNG:OnCommReceive(prefix, sender, distribution, cmd, a1, a2, a3, a4, a5, a6)
	--self:Print("OnCommReceive: "..(cmd or "nil")..", "..(type(a1) == "table" and "table" or a1 or "nil")..", "..(type(a2) == "table" and "table" or a2 or "nil")..", "..(type(a3) == "table" and "table" or a3 or "nil")..", "..(type(a4) == "table" and "table" or a4 or "nil")..", "..(type(a5) == "table" and "table" or a5 or "nil"))
	if not self.board then
		self:DrawBoard()
	end
	if(self.loadInProgress and distribution == "GROUP") then return end

	if(self.syncInProgress and self.syncMax) then
		self.syncCount = self.syncCount + 1
		local p = math.floor((self.syncCount / self.syncMax)*100)
		self.syncFrame.count:SetText(p.."%")
	end

	if(self.loadInProgress) then
		if(cmd ~= "LOADSTOP") then
			self:AddToSync({cmd, a1, a2, a3, a4, a5, a6})
		end
	end

	if(cmd == "DRAWLINE" or cmd == "ERASE" or cmd == "CLEAR" or
	   cmd == "SETTEXT" or cmd == "REMTEXT" or
	   cmd == "SETICON" or cmd == "REMICON") then
		self.dataReceived = true
	end

	-- Line drawing/erasing and board clear
	if(cmd == "DRAWLINE") then
		IMBA_DrawLine(self.board,a1,a2,a3,a4,32,a5)
	elseif(cmd == "ERASE") then
		IMBA_EraseLines(self.board,a1,a2,12)
	elseif(cmd == "CLEAR") then
		self:ClearBoard(true)

	-- Set master / Set master (quiet) / Request master
	elseif(cmd == "SETMASTER") then
		self:SetMaster(a1, true)
	elseif(cmd == "QSETMASTER") then
		self:SetMaster(a1, true, true)
	elseif(cmd == "REQMASTER") then
		if(self:IsLeader(true)) then
			self:SendData({"QSETMASTER", self.db.profile.master}, "ALERT")
		end

	-- Set/remove text
	elseif(cmd == "SETTEXT") then
		self:SetText(a1, a2, a3[1], a3[2], a4, a5, a6, true)
	elseif(cmd == "REMTEXT") then
		self:RemoveText(a1, true)

	-- Set/remove icon
	elseif(cmd == "SETICON") then
		self:SetIcon(a1, a2, a3, true)
	elseif(cmd == "REMICON") then
		self:RemoveIcon(a1, true)

	-- Version query
	elseif(cmd == "VERSION") then
		self:SendData({"VERSIONREPLY", self.version}, "ALERT")
	elseif(cmd == "VERSIONREPLY") then
		if(self.versionQuery) then
			for k,v in pairs(self.versions) do
				if(v[1] == sender) then
					self.versions[k][2] = a1
					break
				end
			end
			self:UpdateScrollBar()
		end

	-- Sync start/stop
	elseif(cmd == "SYNCSTART" and distribution == "GROUP") then
		self.syncMax = a1
		self.syncCount = 0
		self.syncInProgress = true
		self:ClearBoard(true)
		self.syncFrame.text:SetText("Syncing in progress...")
		self.syncFrame.count:SetText("0%")
		self:Print("Remote syncing started")
	elseif(cmd == "SYNCSTOP" and distribution == "GROUP") then
		self:SyncEnabled(self.db.profile.sync_enable)
		self.syncInProgress = nil
		self.syncMax = nil
		self.syncCount = nil
		self:Print("Remote syncing complete")

	-- Load start/stop
	elseif(cmd == "LOADSTART" and distribution == "WHISPER") then
		self.syncMax = a1
		self.syncCount = 0
		self.syncInProgress = true
		self.loadInProgress = true
		self:ClearBoard(true)
		self.syncFrame.text:SetText("Loading in progress...")
		self.syncFrame.count:SetText("0%")
	elseif(cmd == "LOADSTOP" and distribution == "WHISPER") then
		self:SyncEnabled(self.db.profile.sync_enable)
		self.syncInProgress = nil
		self.loadInProgress = nil
		self.syncMax = nil
		self.syncCount = nil
	end
end

function WhiteboardNG:SendData(data, prio)
	if(prio and prio == "ALERT") then
		local cmd, a1, a2, a3, a4, a5, a6 = unpack(data)
		self:SendPrioritizedCommMessage(prio, "GROUP", cmd, a1, a2, a3, a4, a5, a6)
	else
		if(not self.syncInProgress) then
			if(self.db.profile.sync_enable) then
				self:AddToSync(data)
			end
			tinsert(self.dataQueue, data)
		else
			if(self.syncStarted) then
				tinsert(self.dataQueue, data)
			end
		end
	end
end

function WhiteboardNG:ProcessQueue()
	if(table.getn(self.dataQueue) > 0) then
		local cmd, a1, a2, a3, a4, a5, a6 = unpack(self.dataQueue[1])
		self:SendCommMessage("GROUP", cmd, a1, a2, a3, a4, a5, a6)
		tremove(self.dataQueue, 1)
	end
end

function WhiteboardNG:SyncEnabled(state)
	if(not self.syncInProgress) then
		self.syncData = compost:Erase(self.syncData)
	end
	if not self.board then
		return
	end
	if(state) then
		self.syncFrame.text:SetText("Objects in sync-data:")
		self.syncFrame.count:SetText(table.getn(self.syncData))
	else
		self.syncFrame.text:SetText("Syncing not enabled")
		self.syncFrame.count:SetText("")
	end
end

function WhiteboardNG:AddToSync(data)
	tinsert(self.syncData, data)
	self.syncFrame.count:SetText(table.getn(self.syncData))
end

function WhiteboardNG:ClearSync()
	if(self.db.profile.sync_enable) then
		self.syncData = compost:Erase(self.syncData)
		self.syncFrame.count:SetText("0")
	end
end

function WhiteboardNG:DoSync()
	self:Print("Syncing data...")
	self.syncInProgress = true
	self.syncStarted = GetTime()
	self.dataQueue = compost:Erase(self.dataQueue)
	self:SendData({"SYNCSTART", table.getn(self.syncData)}, "ALERT")
	for k,v in pairs(self.syncData) do
		self:SendData(v)
	end
	self:SendData({"SYNCSTOP"})
	self.syncFrame.text:SetText("Syncing data...")
	self:StartMetro("Whiteboard_SyncStatus")
end

function WhiteboardNG:DataQueueStatus()
	local c = table.getn(self.dataQueue)
	if(self.syncInProgress and c == 0 and self.syncStarted and (self.syncStarted + 5) < GetTime()) then
		self:SyncEnabled(self.db.profile.sync_enable)
		self.syncInProgress = false
		self.syncStarted = nil
		self:Print("Syncing is complete")
	end
	if not self.board then
		return
	end
	self.dataFrame.count:SetText(c)
end

function WhiteboardNG:DoVersionQuery()
	self:SendData({"VERSION"}, "ALERT")
	self.versionQuery = true
	if(not self.versions) then
		self.versions = compost:Acquire()
	else
		self.versions = compost:Erase(self.versions)
	end

	if(GetNumRaidMembers() > 0) then
		for i=1,GetNumRaidMembers() do
			tinsert(self.versions, {UnitName("raid"..i), (UnitIsUnit("player", "raid"..i) and self.version or "-")})
		end
	elseif(GetNumPartyMembers() > 0) then
		tinsert(self.versions, {UnitName("player"), self.version})
		for i=1,GetNumPartyMembers() do
			tinsert(self.versions, {UnitName("party"..i), "-"})
		end
	end
	self:UpdateScrollBar()
	self.versionFrame:Show()
end

function WhiteboardNG:CheckDataReceived()
	if not self.board then
		return
	end
	if(self.dataReceived and not self.board:IsVisible()) then
		if(self.mini.flash:IsVisible()) then
			self.mini.flash:Hide()
		else
			self.mini.flash:Show()
		end
		if(not self.dataRecvDone) then
			self.mini.text:SetText("Board updated")
			self:Print("Board has been updated")
			PlaySound("TellMessage")
			self.dataRecvDone = true
		end
	else
		if(self.mini.flash:IsVisible()) then
			self.mini.flash:Hide()
		end
		if(self.dataRecvDone) then
			self.mini.text:SetText("WhiteboardNG")
		end
		self.dataRecvDone = false
	end
end


-- Save/Load/List functions
function WhiteboardNG:SaveDrawing(id)
	if(self.syncInProgress) then return end
	if(not self.db.profile.sync_enable) then return end

	local data = {}
	for k,v in pairs(self.syncData) do
		tinsert(data, v)
	end

	local exists, name = 0, ""
	for k,v in pairs(self.db.profile.drawings) do
		if(string.lower(id) == string.lower(v.name)) then
			exists = k
			name = v.name
		end
	end

	if(exists == 0) then
		name = id
		tinsert(self.db.profile.drawings, {["name"] = name, ["data"] = data})
	else
		tremove(self.db.profile.drawings, exists)
		tinsert(self.db.profile.drawings, exists, {["name"] = name, ["data"] = data})
	end

	self:Print("Saved drawing '"..name.."'")
end

function WhiteboardNG:LoadDrawing(id)
	if(self.syncInProgress) then return end
	if(not self.db.profile.sync_enable) then return end

	local data = {}
	for k,v in pairs(self.db.profile.drawings) do
		if(string.lower(id) == string.lower(v.name)) then
			data = v.data
		end
	end
	if(table.getn(data) == 0) then
		self:Print("No drawing by that name")
		return
	end

	self:OnCommReceive("", "", "WHISPER", "LOADSTART", table.getn(data))
	for k,v in pairs(data) do
		local cmd, a1, a2, a3, a4, a5, a6 = unpack(v)
		self:OnCommReceive("", "", "WHISPER", cmd, a1, a2, a3, a4, a5, a6)
	end
	self:OnCommReceive("", "", "WHISPER", "LOADSTOP")

	self:Print("Loaded drawing '"..id.."'")
end

function WhiteboardNG:ListDrawings()
	self:Print("Listing saved drawings:")
	for k,v in pairs(self.db.profile.drawings) do
		self:Print("- ".. v.name)
	end
	self:Print("Done")
end

function WhiteboardNG:DeleteDrawing(id)
	local exists, name = 0, ""
	for k,v in pairs(self.db.profile.drawings) do
		if(string.lower(id) == string.lower(v.name)) then
			exists = k
			name = v.name
		end
	end
	if(exists == 0) then
		self:Print("No drawing by that name")
		return
	else
		self:Print("Drawing '"..name.."' removed")
		tremove(self.db.profile.drawings, exists)
	end
end

-- Helper functions
function WhiteboardNG:InGroup()
	if(GetNumRaidMembers() > 0) then
		return true
	elseif(GetNumPartyMembers() > 0) then
		return true
	end
	return false
end

function WhiteboardNG:IsLeader(noofficer)
	if(IsRaidLeader() or (IsRaidOfficer() and not noofficer) or IsPartyLeader()) then
		return true
	end
	return false
end

function WhiteboardNG:OnBoard(X, Y)
	X = X + 2
	Y = Y + 2
	if(X > (self.board:GetWidth() - 4)) or (X < 0) or (Y > (self.board:GetHeight() - 4)) or (Y < 0) then
		return false
	end
	return true
end

function WhiteboardNG:AllowedDraw()
	if(self.syncing) then
		return false
	end

	if(self.db.profile.master == "<off>") then
		return true
	elseif(self.db.profile.master == "<leadersonly>" and self:IsLeader()) then
		return true
	elseif(self.db.profile.master == string.lower(UnitName("player"))) then
		return true
	end
	return false
end

function WhiteboardNG:ScrollEntryFrame()
	local f = CreateFrame("Button", nil, self.versionFrame)
	f:SetWidth(180)
	f:SetHeight(16)

	f.name = f:CreateFontString(nil,"ARTWORK")
	f.name:SetFontObject(GameFontHighlight)
	f.name:SetWidth(130)
	f.name:SetHeight(16)
	f.name:SetJustifyH("LEFT")
	f.name:SetNonSpaceWrap(false)
	f.name:ClearAllPoints()
	f.name:SetPoint( "LEFT", f, "LEFT")
	f.name:SetTextColor(1,1,1)
	f.name:SetText("Name")

	f.version = f:CreateFontString(nil,"ARTWORK")
	f.version:SetFontObject(GameFontHighlight)
	f.version:SetWidth(50)
	f.version:SetHeight(16)
	f.version:SetJustifyH("LEFT")
	f.version:SetNonSpaceWrap(false)
	f.version:ClearAllPoints()
	f.version:SetPoint("LEFT", f.name, "RIGHT")
	f.version:SetTextColor(1,1,1)
	f.version:SetText("0.0.0")

	return f
end

function WhiteboardNG:UpdateScrollBar()
	local i,j
	local entries = table.getn(self.versions)
	FauxScrollFrame_Update(self.versionFrame.sframe, entries, 27, 28)

	for i = 1, 27 do
		j = i + FauxScrollFrame_GetOffset(self.versionFrame.sframe)

		if j <= entries then
			self.versionFrame.entry[i].name:SetText(self.versions[j][1])
			self.versionFrame.entry[i].version:SetText(self.versions[j][2])
			if(self.versions[j][2] == self.version or self.versions[j][2] == "-") then
				self.versionFrame.entry[i].version:SetTextColor(1, 1, 1, 1)
			elseif(self.versions[j][2] < self.version) then
				self.versionFrame.entry[i].version:SetTextColor(1, 0, 0, 1)
			else
				self.versionFrame.entry[i].version:SetTextColor(0, 1, 0, 1)
			end
			self.versionFrame.entry[i]:Show()
		else
			self.versionFrame.entry[i]:Hide()
		end
	end
end


-- Dewdrop menu
dewdropmenu = {
	type = "group",
	name = "WhiteboardNG",
	args = {
		header1 = {
			name = "Options",
			type = "header",
			order = 1,
		},
		master = {
			name = "Master drawer",
			desc = "Set options relating to master drawer",
			type = "group",
			order = 2,
			disabled = function() return (not WhiteboardNG:InGroup() or not WhiteboardNG:IsLeader()) end,
			args = {
				off = {
					name = "Off",
					desc = "Turn master drawer off",
					type = "execute",
					func = function() WhiteboardNG:SetMaster("<off>") end,
				},
				leadersonly = {
					name = "Leaders only",
					desc = "Set raid leader + assistants/party leader as master",
					type = "execute",
					func = function() WhiteboardNG:SetMaster("<leadersonly>") end,
				},
				set = {
					name = "Set master",
					desc = "Set master drawer",
					type = "text",
					set = function(a) WhiteboardNG:SetMaster(a) end,
					get = function() return WhiteboardNG.db.profile.master end,
					usage = "<name>",
					validate = function(a)
						if not a then
							return WhiteboardNG:SetMaster("<off>")
						end
						if(GetNumRaidMembers() > 0) then
							for i=1,40 do
								if(string.lower(UnitName("raid"..i)) == string.lower(a)) then
									return true
								end
							end
						elseif(GetNumPartyMembers() > 0) then
							for i=1,4 do
								if(string.lower(UnitName("party"..i)) == string.lower(a)) then
									return true
								end
							end
						end
						return false
					end
				},
			},
		},
		sync = {
			name = "Syncing",
			desc = "Options relating to syncing",
			type = "group",
			order = 3,
			disabled = function() return not WhiteboardNG:AllowedDraw() end,
			args = {
				enabled = {
					name = "Enabled",
					desc = "Data-collection for syncing",
					type = "toggle",
					get = function() return WhiteboardNG.db.profile.sync_enable end,
					set = function(a)
						WhiteboardNG.db.profile.sync_enable = a
						WhiteboardNG:SyncEnabled(a)
					end,
				},
				syncnow = {
					name = "Sync Now!",
					desc = "Sync current data to other users",
					type = "execute",
					func = function() WhiteboardNG:DoSync() end,
					disabled = function()
						return (not WhiteboardNG:InGroup() or not WhiteboardNG.db.profile.sync_enable or
						(WhiteboardNG.syncData and table.getn(WhiteboardNG.syncData) == 0))
					end,
				},
			},
		},
		scale = {
			name = "Window scale",
			desc = "Set whiteboard window scale",
			type = "range",
			min = 0.25,
			max = 2,
			step = 0.05,
			isPercent = true,
			get = function() return WhiteboardNG.board:GetScale() end,
			set = function(scale) WhiteboardNG.board:SetScale(scale) end,
			order = 4,
		},
		sep1 = {
			type = "header",
			order = 10,
		},
		header2 = {
			name = "Whiteboard",
			type = "header",
			order = 11,
		},
		color = {
			name = "Color",
			desc = "Change color",
			type = "color",
			get = function()
				return unpack(WhiteboardNG.optColor)
			end,
			set = function(r, g, b, a)
				WhiteboardNG.optColor = {r, g, b, a}
			end,
			hasAlpha = true,
			order = 12,
		},
		icons = {
			name = "Raid icons",
			desc = "Add a raid icon",
			type = "group",
			order = 13,
			disabled = function() return not WhiteboardNG:AllowedDraw() end,
			args = {
			},
		},
		text = {
			name = "Add text",
			desc = "Add a new text object (right-click text pin for more options)",
			type = "execute",
			func = function() WhiteboardNG:SetText(-1, nil, -1, -1, nil, nil, "Text") end,
			order = 14,
			disabled = function() return not WhiteboardNG:AllowedDraw() end,
		},
		sep2 = {
			name = "  ",
			type = "header",
			order = 20,
		},
		savedrawing = {
			name = "Save drawing",
			desc = "Save the current sync-data",
			type = "text",
			get = false,
			set = function(id) WhiteboardNG:SaveDrawing(id) end,
			validate = function(id) return true end,
			usage = "<id>",
			order = 21,
			disabled = function() return not WhiteboardNG.db.profile.sync_enable or
				(WhiteboardNG.syncData and table.getn(WhiteboardNG.syncData) == 0) end,
		},
		loaddrawing = {
			name = "Load drawing",
			desc = "Load a drawing into sync-data",
			type = "text",
			get = false,
			set = function(id) WhiteboardNG:LoadDrawing(id) end,
			usage = "<id>",
			order = 22,
			disabled = function() return not WhiteboardNG.db.profile.sync_enable or (table.getn(WhiteboardNG.db.profile.drawings) == 0) end,
		},
		listdrawings = {
			name = "List drawings",
			desc = "List all previously saved drawings",
			type = "execute",
			func = function() WhiteboardNG:ListDrawings() end,
			order = 23,
			disabled = function() return (table.getn(WhiteboardNG.db.profile.drawings) == 0) end,
		},
		deldrawing = {
			name = "Remove drawing",
			desc = "Remove the selected drawing",
			type = "text",
			get = false,
			set = function(id) WhiteboardNG:DeleteDrawing(id) end,
			usage = "<id>",
			order = 24,
			disabled = function() return (table.getn(WhiteboardNG.db.profile.drawings) == 0) end,
		},
		sep3 = {
			name = "  ",
			type = "header",
			order = 30,
		},
		version = {
			name = "Version query",
			desc = "Query everyone in the party/raid for Whiteboard version",
			type = "execute",
			func = function() WhiteboardNG:DoVersionQuery() end,
			disabled = function() return not WhiteboardNG:InGroup() end,
			order = 31,
		},
		clear = {
			name = "Clear board",
			desc = "Clear the whiteboard of all drawings",
			type = "execute",
			func = function() WhiteboardNG:ClearBoard() end,
			disabled = function() return not WhiteboardNG:AllowedDraw() end,
			order = 32,
		},
		minimize = {
			name = "Minimize window",
			desc = "Minimize the whiteboard window",
			type = "execute",
			func = function()
				WhiteboardNG.board:Hide()
				WhiteboardNG.mini:Show()
			end,
			order = 34,
		},
		close = {
			name = "Close window",
			desc = "Close the whiteboard window",
			type = "execute",
			func = function()
				WhiteboardNG.board:Hide()
			end,
			order = 35,
		},
	},
}
WhiteboardNG.Icons = {
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Star.tga", ID = "Star"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Circle.tga", ID = "Circle"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Diamond.tga", ID = "Diamond"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Triangle.tga", ID = "Triangle"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Moon.tga", ID = "Moon"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Square.tga", ID = "Square"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Cross.tga", ID = "Cross"},
	{Texture = "Interface\\AddOns\\WhiteboardNG\\textures\\Skull.tga", ID = "Skull"},
}
local tmp = {}
for k,v in pairs(WhiteboardNG.Icons) do
	tmp = {
		name = v.ID,
		desc = "Add a "..string.lower(v.ID),
		type = "execute",
		order = k,
		icon = v.Texture,
	}
	dewdropmenu.args.icons.args[string.lower(v.ID)] = tmp
end
dewdropmenu.args.icons.args.star.func = function() WhiteboardNG:SetIcon(1, -1, -1) end
dewdropmenu.args.icons.args.circle.func = function() WhiteboardNG:SetIcon(2, -1, -1) end
dewdropmenu.args.icons.args.diamond.func = function() WhiteboardNG:SetIcon(3, -1, -1) end
dewdropmenu.args.icons.args.triangle.func = function() WhiteboardNG:SetIcon(4, -1, -1) end
dewdropmenu.args.icons.args.moon.func = function() WhiteboardNG:SetIcon(5, -1, -1) end
dewdropmenu.args.icons.args.square.func = function() WhiteboardNG:SetIcon(6, -1, -1) end
dewdropmenu.args.icons.args.cross.func = function() WhiteboardNG:SetIcon(7, -1, -1) end
dewdropmenu.args.icons.args.skull.func = function() WhiteboardNG:SetIcon(8, -1, -1) end

dewdropmenu.args.icons.args.star.disabled = function() return frames.icons.Star and frames.icons.Star.active end
dewdropmenu.args.icons.args.circle.disabled = function() return frames.icons.Circle and frames.icons.Circle.active  end
dewdropmenu.args.icons.args.diamond.disabled = function() return frames.icons.Diamond and frames.icons.Diamond.active end
dewdropmenu.args.icons.args.triangle.disabled = function() return frames.icons.Triangle and frames.icons.Triangle.active end
dewdropmenu.args.icons.args.moon.disabled = function() return frames.icons.Moon and frames.icons.Moon.active end
dewdropmenu.args.icons.args.square.disabled = function() return frames.icons.Square and frames.icons.Square.active end
dewdropmenu.args.icons.args.cross.disabled = function() return frames.icons.Cross and frames.icons.Cross.active end
dewdropmenu.args.icons.args.skull.disabled = function() return frames.icons.Skull and frames.icons.Skull.active end


WhiteboardNG.pingIcons = {
	{0, 0.25, 0, 0.25},
	{0.25, 0.50, 0, 0.25},
	{0.50, 0.75, 0, 0.25},
	{0, 0.25, 0.25, 0.5}
}
textmenu = {
	type = "group",
	name = "WhiteboardNG",
	args = {
		header1 = {
			name = "Text Object",
			type = "header",
			order = 1,
		},
		settext = {
			name = "Set text",
			desc = "Set the text to be displayed",
			type = "text",
			get = function()
				local parent = dewdrop:GetOpenedParent()
				return parent.txt:GetText()
			end,
			set = function(msg)
				local parent = dewdrop:GetOpenedParent()
				parent.txt:SetText(msg)
				WhiteboardNG:SendText(parent.id)
			end,
			usage = "<text>",
			order = 3,
		},
		color = {
			name = "Text color",
			desc = "Set the color of the text",
			type = "color",
			get = function()
				local parent = dewdrop:GetOpenedParent()
				return parent.txt:GetTextColor()
			end,
			set = function(r, g, b, a)
				curtextmenu.txt:SetTextColor(r, g, b, a)
				if(not ColorPickerFrame:IsVisible()) then
					WhiteboardNG:SendText(curtextmenu.id)
				end
			end,
			hasAlpha = true,
			order = 4,
		},
		justify = {
			name = "Justify",
			desc = "Set how the text should be justified",
			type = "text",
			get = function()
				local parent = dewdrop:GetOpenedParent()
				return parent.txt:GetJustifyH()
			end,
			set = function(justify)
				local parent = dewdrop:GetOpenedParent()
				parent.txt:SetJustifyH(justify)

				parent.txt:ClearAllPoints()
				if(justify == "LEFT") then
					parent.txt:SetPoint("BOTTOMLEFT", parent, "TOPRIGHT")
				elseif(justify == "CENTER") then
					parent.txt:SetPoint("CENTER", parent, "TOPLEFT", parent:GetWidth()/2, 12)
				elseif(justify == "RIGHT") then
					parent.txt:SetPoint("BOTTOMRIGHT", parent, "TOPLEFT")
				end
				WhiteboardNG:SendText(parent.id)
			end,
			validate = {"LEFT", "CENTER", "RIGHT"},
			order = 5,
		},
		ping = {
			name = "Set icon",
			desc = "Set the icon the text is attached to",
			type = "text",
			get = function()
				local parent = dewdrop:GetOpenedParent()
				local ping = {"Yellow", "Red", "Green", "Blue"}
				return ping[parent.ping]
			end,
			set = function(color)
				local parent = dewdrop:GetOpenedParent()
				local ping = {["Yellow"] = 1, ["Red"] = 2, ["Green"] = 3, ["Blue"] = 4}
				parent.ping = ping[color]
				parent.texture:SetTexCoord(unpack(WhiteboardNG.pingIcons[parent.ping]))
				WhiteboardNG:SendText(parent.id)
			end,
			validate = {"Yellow", "Red", "Green", "Blue"},
			order = 6,
		},
		sep1 = {
			name = "  ",
			type = "header",
			order = 7,
		},
		delete = {
			name = "Remove text",
			desc = "Remove the selected text object",
			type = "execute",
			func = function()
				local parent = dewdrop:GetOpenedParent()
				WhiteboardNG:RemoveText(parent.id)
				dewdrop:Close()
			end,
			order = 8,
		},
	},
}
